@@ -26,14 +26,18 @@
  *    Rob Clark <robclark@freedesktop.org>
  */
 
+#ifdef HAVE_CONFIG_H
+# include <config.h>
+#endif
+
 #include <assert.h>
 
 #include "freedreno_drmif.h"
 #include "freedreno_priv.h"
 #include "freedreno_ringbuffer.h"
 
-struct fd_ringbuffer * fd_ringbuffer_new(struct fd_pipe *pipe,
-		uint32_t size)
+struct fd_ringbuffer *
+fd_ringbuffer_new(struct fd_pipe *pipe, uint32_t size)
 {
 	struct fd_ringbuffer *ring;
 
@@ -41,10 +45,9 @@ struct fd_ringbuffer * fd_ringbuffer_new(struct fd_pipe *pipe,
 	if (!ring)
 		return NULL;
 
-	ring->size = size;
 	ring->pipe = pipe;
 	ring->start = ring->funcs->hostptr(ring);
-	ring->end = &(ring->start[size/4]);
+	ring->end = &(ring->start[ring->size/4]);
 
 	ring->cur = ring->last_start = ring->start;
 
@@ -53,6 +56,7 @@ struct fd_ringbuffer * fd_ringbuffer_new(struct fd_pipe *pipe,
 
 void fd_ringbuffer_del(struct fd_ringbuffer *ring)
 {
+	fd_ringbuffer_reset(ring);
 	ring->funcs->destroy(ring);
 }
 
@@ -61,7 +65,7 @@ void fd_ringbuffer_del(struct fd_ringbuffer *ring)
  * the bookkeeping works out properly.
  */
 void fd_ringbuffer_set_parent(struct fd_ringbuffer *ring,
-		struct fd_ringbuffer *parent)
+					 struct fd_ringbuffer *parent)
 {
 	ring->parent = parent;
 }
@@ -76,10 +80,31 @@ void fd_ringbuffer_reset(struct fd_ringbuffer *ring)
 		ring->funcs->reset(ring);
 }
 
-/* maybe get rid of this and use fd_ringmarker_flush() from DDX too? */
 int fd_ringbuffer_flush(struct fd_ringbuffer *ring)
 {
-	return ring->funcs->flush(ring, ring->last_start);
+	return ring->funcs->flush(ring, ring->last_start, -1, NULL);
+}
+
+int fd_ringbuffer_flush2(struct fd_ringbuffer *ring, int in_fence_fd,
+		int *out_fence_fd)
+{
+	return ring->funcs->flush(ring, ring->last_start, in_fence_fd, out_fence_fd);
+}
+
+void fd_ringbuffer_grow(struct fd_ringbuffer *ring, uint32_t ndwords)
+{
+	assert(ring->funcs->grow);     /* unsupported on kgsl */
+
+	/* there is an upper bound on IB size, which appears to be 0x100000 */
+	if (ring->size < 0x100000)
+		ring->size *= 2;
+
+	ring->funcs->grow(ring, ring->size);
+
+	ring->start = ring->funcs->hostptr(ring);
+	ring->end = &(ring->start[ring->size/4]);
+
+	ring->cur = ring->last_start = ring->start;
 }
 
 uint32_t fd_ringbuffer_timestamp(struct fd_ringbuffer *ring)
@@ -88,7 +113,14 @@ uint32_t fd_ringbuffer_timestamp(struct fd_ringbuffer *ring)
 }
 
 void fd_ringbuffer_reloc(struct fd_ringbuffer *ring,
-		const struct fd_reloc *reloc)
+				    const struct fd_reloc *reloc)
+{
+	assert(ring->pipe->gpu_id < 500);
+	ring->funcs->emit_reloc(ring, reloc);
+}
+
+void fd_ringbuffer_reloc2(struct fd_ringbuffer *ring,
+				     const struct fd_reloc *reloc)
 {
 	ring->funcs->emit_reloc(ring, reloc);
 }
@@ -96,8 +128,31 @@ void fd_ringbuffer_reloc(struct fd_ringbuffer *ring,
 void fd_ringbuffer_emit_reloc_ring(struct fd_ringbuffer *ring,
 		struct fd_ringmarker *target, struct fd_ringmarker *end)
 {
+	uint32_t submit_offset, size;
+
+	/* This function is deprecated and not supported on 64b devices: */
+	assert(ring->pipe->gpu_id < 500);
 	assert(target->ring == end->ring);
-	ring->funcs->emit_reloc_ring(ring, target, end);
+
+	submit_offset = offset_bytes(target->cur, target->ring->start);
+	size = offset_bytes(end->cur, target->cur);
+
+	ring->funcs->emit_reloc_ring(ring, target->ring, 0, submit_offset, size);
+}
+
+uint32_t fd_ringbuffer_cmd_count(struct fd_ringbuffer *ring)
+{
+	if (!ring->funcs->cmd_count)
+		return 1;
+	return ring->funcs->cmd_count(ring);
+}
+
+uint32_t
+fd_ringbuffer_emit_reloc_ring_full(struct fd_ringbuffer *ring,
+		struct fd_ringbuffer *target, uint32_t cmd_idx)
+{
+	uint32_t size = offset_bytes(target->cur, target->start);
+	return ring->funcs->emit_reloc_ring(ring, target, cmd_idx, 0, size);
 }
 
 struct fd_ringmarker * fd_ringmarker_new(struct fd_ringbuffer *ring)
@@ -112,7 +167,7 @@ struct fd_ringmarker * fd_ringmarker_new(struct fd_ringbuffer *ring)
 
 	marker->ring = ring;
 
-	fd_ringmarker_mark(marker);
+	marker->cur = marker->ring->cur;
 
 	return marker;
 }
@@ -128,7 +183,7 @@ void fd_ringmarker_mark(struct fd_ringmarker *marker)
 }
 
 uint32_t fd_ringmarker_dwords(struct fd_ringmarker *start,
-		struct fd_ringmarker *end)
+					 struct fd_ringmarker *end)
 {
 	return end->cur - start->cur;
 }
@@ -136,5 +191,5 @@ uint32_t fd_ringmarker_dwords(struct fd_ringmarker *start,
 int fd_ringmarker_flush(struct fd_ringmarker *marker)
 {
 	struct fd_ringbuffer *ring = marker->ring;
-	return ring->funcs->flush(ring, marker->cur);
+	return ring->funcs->flush(ring, marker->cur, -1, NULL);
 }
