@@ -39,6 +39,8 @@
 
 struct msm_device {
 	struct fd_device base;
+	struct fd_bo_cache ring_cache;
+	unsigned ring_cnt;
 };
 
 static inline struct msm_device * to_msm_device(struct fd_device *x)
@@ -54,6 +56,7 @@ struct msm_pipe {
 	uint32_t gpu_id;
 	uint32_t gmem;
 	uint32_t chip_id;
+	uint32_t queue_id;
 };
 
 static inline struct msm_pipe * to_msm_pipe(struct fd_pipe *x)
@@ -62,7 +65,7 @@ static inline struct msm_pipe * to_msm_pipe(struct fd_pipe *x)
 }
 
 drm_private struct fd_pipe * msm_pipe_new(struct fd_device *dev,
-		enum fd_pipe_id id);
+		enum fd_pipe_id id, uint32_t prio);
 
 drm_private struct fd_ringbuffer * msm_ringbuffer_new(struct fd_pipe *pipe,
 		uint32_t size);
@@ -71,18 +74,11 @@ struct msm_bo {
 	struct fd_bo base;
 	uint64_t offset;
 	uint64_t presumed;
-	/* in the common case, a bo won't be referenced by more than a single
-	 * (parent) ring[*].  So to avoid looping over all the bo's in the
-	 * reloc table to find the idx of a bo that might already be in the
-	 * table, we cache the idx in the bo.  But in order to detect the
-	 * slow-path where bo is ref'd in multiple rb's, we also must track
-	 * the current_ring for which the idx is valid.  See bo2idx().
-	 *
-	 * [*] in case multiple ringbuffers, ie. one toplevel and other rb(s)
-	 *     used for IB target(s), the toplevel rb is the parent which is
-	 *     tracking bo's for the submit
+	/* to avoid excess hashtable lookups, cache the ring this bo was
+	 * last emitted on (since that will probably also be the next ring
+	 * it is emitted on)
 	 */
-	struct fd_ringbuffer *current_ring;
+	unsigned current_ring_seqno;
 	uint32_t idx;
 };
 
